------------------- *** SQL Server 2012 spatial enhancements *** -------------------

-- Three new curved shapes:
--	Circular strings
--	Compound curves
--	Curve polygons


-- *** Circular Strings ***

-- Simple circular string
SELECT geometry::Parse('CIRCULARSTRING(0 1, .25 0, 0 -1)')

-- Straight line
SELECT geometry::Parse('LINESTRING(0 8, 8 -8)')

-- Take the same straight line and curve it
SELECT geometry::Parse('CIRCULARSTRING(0 8, 4 0, 8 -8)').STBuffer(.1)
UNION ALL  -- Curve it
SELECT geometry::Parse('CIRCULARSTRING(0 8, 4 4, 8 -8)').STBuffer(.1)
UNION ALL  -- Curve it some more
SELECT geometry::Parse('CIRCULARSTRING(0 8, 4 6, 8 -8)').STBuffer(.1)
UNION ALL  -- Curve it in the other direction
SELECT geometry::Parse('CIRCULARSTRING(0 8, 4 -6, 8 -8)').STBuffer(.1)

-- Link two semi-circles together to form a circle
SELECT geometry::Parse('CIRCULARSTRING(0 4, 4 0, 8 4, 4 8, 0 4)')

-- Link two circular strings together to form a binoculars shape
SELECT geometry::Parse('CIRCULARSTRING(0 1, 5 0, 0 -2, -5 0, 0 1)')

-- Collapsed
SELECT geometry::Parse('CIRCULARSTRING(0 5, 2 0, 0 -5, -2 0, 0 5)')


-- *** Compound Curves ***

-- Similar to a geometry collection, but:
--  allows only line strings and circular strings
--  requires connecting end-start points
--  more space-efficient

-- Compound curve
DECLARE @CC geometry = '
 COMPOUNDCURVE(
  (4 4, 4 8),
  CIRCULARSTRING(4 8, 6 10, 8 8),
  (8 8, 8 4),
  CIRCULARSTRING(8 4, 2 3, 4 4)
 )'

-- Equivalent geometry collection
DECLARE @GC geometry = '
 GEOMETRYCOLLECTION(
  LINESTRING(4 4, 4 8),
  CIRCULARSTRING(4 8, 6 10, 8 8),
  LINESTRING(8 8, 8 4),
  CIRCULARSTRING(8 4, 2 3, 4 4)
 )'

-- They both render the same shape in the spatial viewer
SELECT @CC.STBuffer(.5)
UNION ALL
SELECT @GC.STBuffer(1.5)

-- They both contain the exact same set of points
SELECT @CC.STEquals(@GC)

-- The compound curve takes only 152 bytes
-- while the geometry collection takes 243
-- so it's 91 bytes less (roughly 43% savings)
SELECT DATALENGTH(@CC), DATALENGTH(@GC)
GO


-- *** Curve Polygons ***

SELECT geometry::Parse('
  CURVEPOLYGON(
   COMPOUNDCURVE(
    (4 4, 4 8),
    CIRCULARSTRING(4 8, 6 10, 8 8),
    (8 8, 8 4),
    CIRCULARSTRING(8 4, 2 3, 4 4)
   )
  )')
GO


-- *** STNumCurves and STCurveN *** --

-- Create a full circle shape (two connected semi-circles)
DECLARE @C geometry = 'CIRCULARSTRING(0 4, 4 0, 8 4, 4 8, 0 4)'

-- Get the curve count (2) and the 1st curve (bottom semi-circle)
SELECT
	CurveCount = @C.STNumCurves(),
	SecondCurve = @C.STCurveN(1)

GO


-- *** BufferWithCurves ***

DECLARE @streets geometry = '
 GEOMETRYCOLLECTION(
  LINESTRING (100 -100, 20 -180, 180 -180),
  LINESTRING (300 -300, 300 -150, 50 -50)
 )'
SELECT @streets.BufferWithCurves(10)

SELECT
  AsWKT = @streets.ToString(),
  Bytes = DATALENGTH(@streets),
  Points = @streets.STNumPoints()
 UNION ALL
 SELECT
  @streets.STBuffer(10).ToString(),
  DATALENGTH(@streets.STBuffer(10)),
  @streets.STBuffer(10).STNumPoints()
 UNION ALL
 SELECT
  @streets.BufferWithCurves(10).ToString(),
  DATALENGTH(@streets.BufferWithCurves(10)),
  @streets.BufferWithCurves(10).STNumPoints()


-- *** ShortestLineTo *** --

DECLARE @Shape1 geometry = '
 POLYGON ((-20 -30, -3 -26, 14 -28, 20 -40, -20 -30))'

DECLARE @Shape2 geometry = '
 POLYGON ((-18 -20, 0 -10, 4 -12, 10 -20, 2 -22, -18 -20))'

SELECT @Shape1
UNION ALL
SELECT @Shape2
UNION ALL
SELECT @Shape1.ShortestLineTo(@Shape2).STBuffer(.25)
GO


-- *** MinDbCompatibilityLevel

DECLARE @Shape1 geometry = 'CIRCULARSTRING(0 50, 90 50, 180 50)'
DECLARE @Shape2 geometry = 'LINESTRING (0 50, 90 50, 180 50)'

SELECT
 Shape1MinVersion = @Shape1.MinDbCompatibilityLevel(),
 Shape2MinVersion = @Shape2.MinDbCompatibilityLevel()


-- *** STCurveToLine and CurveToLineWithTolerance

-- Create a full circle shape (two connected semi-circles)
DECLARE @C geometry = 'CIRCULARSTRING(0 4, 4 0, 8 4, 4 8, 0 4)'

-- Render as curved shape
SELECT
  Shape = @C,
  ShapeWKT = @C.ToString(),
  ShapeLen = DATALENGTH(@C),
  Points = @C.STNumPoints()

-- Convert to lines (much larger, many more points)
SELECT
  Shape = @C.STCurveToLine(),
  ShapeWKT = @C.STCurveToLine().ToString(),
  ShapeLen = DATALENGTH(@C.STCurveToLine()),
  Points = @C.STCurveToLine().STNumPoints()

-- Convert to lines with tolerance (not as much larger, not as many more points)
SELECT
  Shape = @C.CurveToLineWithTolerance(0.1, 0),
  ShapeWKT = @C.CurveToLineWithTolerance(0.1, 0).ToString(),
  ShapeLen = DATALENGTH(@C.CurveToLineWithTolerance(0.1, 0)),
  Points = @C.CurveToLineWithTolerance(0.1, 0).STNumPoints()

GO


-- *** STIsValid, STIsValidDetailed, and MakeValid ***

DECLARE @line geometry = 'LINESTRING(1 1, 2 2, 3 2, 2 2)'
SELECT
 IsValid = @line.STIsValid(),
 Details = @line.IsValidDetailed()
GO

-- You can still perform metric operations on invalid objects...
DECLARE @line geometry = 'LINESTRING(1 1, 2 2, 3 2, 2 2)'
SELECT @line.STLength()
GO

-- ...but not other operations
DECLARE @line geometry = 'LINESTRING(1 1, 2 2, 3 2, 2 2)'
SELECT @line.STBuffer(.1)
GO

-- Make it valid (changes the shape)
DECLARE @line geometry = 'LINESTRING(1 1, 2 2, 3 2, 2 2)'
SELECT @line.MakeValid().ToString()
GO


-- *** Exceed Logical Hemisphere ***

-- Small (less than a logical hemisphere) polygon
SELECT geography::Parse('POLYGON((-10 -10, 10 -10, 10 10, -10 10, -10 -10))')

-- Reorder in the opposite direction for "rest of the globe"
SELECT geography::Parse('POLYGON((-10 -10, -10 10, 10 10, 10 -10, -10 -10))')

-- Reorient back to the small polygon
SELECT geography::Parse('POLYGON((-10 -10, -10 10, 10 10, 10 -10, -10 -10))').ReorientObject()

GO


-- *** Full Globe ***

-- Construct a new FullGlobe object (a WGS84 ellipsoid)
DECLARE @Earth geography = 'FULLGLOBE'

-- Calculate the area of the earth (510,065,621,710,996 square meters)
SELECT PlanetArea = @Earth.STArea()
